@@ -16,6 +16,7 @@ import argparse
 import copy
 import os
 
+import extras
 import fixtures
 import testtools
 import yaml
@@ -354,6 +355,66 @@ class TestConfig(base.TestCase):
         self.assertEqual(region, {'name': 'no-cloud-region', 'values': {}})
 
 
+class TestExcludedFormattedConfigValue(base.TestCase):
+    # verify LaunchPad bug #1635696
+    #
+    # get_one_cloud() and get_one_cloud_osc() iterate over config
+    # values and try to expand any variables in those values by
+    # calling value.format(), however some config values
+    # (e.g. password) should never have format() applied to them, not
+    # only might that change the password but it will also cause the
+    # format() function to raise an exception if it can not parse the
+    # format string. Examples would be single brace (e.g. 'foo{')
+    # which raises an ValueError because it's looking for a matching
+    # end brace or a brace pair with a key value that cannot be found
+    # (e.g. 'foo{bar}') which raises a KeyError.
+
+    def setUp(self):
+        super(TestExcludedFormattedConfigValue, self).setUp()
+
+        self.args = dict(
+            auth_url='http://example.com/v2',
+            username='user',
+            project_name='project',
+            region_name='region2',
+            snack_type='cookie',
+            os_auth_token='no-good-things',
+        )
+
+        self.options = argparse.Namespace(**self.args)
+
+    def test_get_one_cloud_password_brace(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+
+        password = 'foo{'       # Would raise ValueError, single brace
+        self.options.password = password
+        cc = c.get_one_cloud(
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
+        self.assertEqual(cc.password, password)
+
+        password = 'foo{bar}'   # Would raise KeyError, 'bar' not found
+        self.options.password = password
+        cc = c.get_one_cloud(
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
+        self.assertEqual(cc.password, password)
+
+    def test_get_one_cloud_osc_password_brace(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        password = 'foo{'       # Would raise ValueError, single brace
+        self.options.password = password
+        cc = c.get_one_cloud_osc(
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
+        self.assertEqual(cc.password, password)
+
+        password = 'foo{bar}'   # Would raise KeyError, 'bar' not found
+        self.options.password = password
+        cc = c.get_one_cloud_osc(
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
+        self.assertEqual(cc.password, password)
+
+
 class TestConfigArgparse(base.TestCase):
 
     def setUp(self):
@@ -366,6 +427,7 @@ class TestConfigArgparse(base.TestCase):
             project_name='project',
             region_name='region2',
             snack_type='cookie',
+            os_auth_token='no-good-things',
         )
 
         self.options = argparse.Namespace(**self.args)
@@ -383,7 +445,7 @@ class TestConfigArgparse(base.TestCase):
                                    vendor_files=[self.vendor_yaml])
 
         cc = c.get_one_cloud(
-            cloud='_test_cloud_regions', argparse=self.options)
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
 
@@ -416,7 +478,7 @@ class TestConfigArgparse(base.TestCase):
         cc = c.get_one_cloud(
             argparse=options, **kwargs)
         self.assertEqual(cc.region_name, 'region2')
-        self.assertEqual(cc.auth['password'], 'argpass')
+        self.assertEqual(cc.auth['password'], 'authpass')
         self.assertEqual(cc.snack_type, 'cookie')
 
     def test_get_one_cloud_precedence_osc(self):
@@ -473,14 +535,14 @@ class TestConfigArgparse(base.TestCase):
 
         cc = c.get_one_cloud(**kwargs)
         self.assertEqual(cc.region_name, 'kwarg_region')
-        self.assertEqual(cc.auth['password'], 'ansible_password')
+        self.assertEqual(cc.auth['password'], 'authpass')
         self.assertIsNone(cc.password)
 
     def test_get_one_cloud_just_argparse(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
 
-        cc = c.get_one_cloud(argparse=self.options)
+        cc = c.get_one_cloud(argparse=self.options, validate=False)
         self.assertIsNone(cc.cloud)
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
@@ -489,7 +551,7 @@ class TestConfigArgparse(base.TestCase):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
 
-        cc = c.get_one_cloud(**self.args)
+        cc = c.get_one_cloud(validate=False, **self.args)
         self.assertIsNone(cc.cloud)
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
@@ -576,6 +638,37 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual('my-network', cc.config['external_network'])
 
+    def test_get_one_cloud_no_yaml_no_cloud(self):
+        c = config.OpenStackConfig(load_yaml_config=False)
+
+        self.assertRaises(
+            exceptions.OpenStackConfigException,
+            c.get_one_cloud,
+            cloud='_test_cloud_regions', region_name='region2', argparse=None)
+
+    def test_get_one_cloud_no_yaml(self):
+        c = config.OpenStackConfig(load_yaml_config=False)
+
+        cc = c.get_one_cloud(
+            region_name='region2', argparse=None,
+            **base.USER_CONF['clouds']['_test_cloud_regions'])
+        # Not using assert_cloud_details because of cache settings which
+        # are not present without the file
+        self.assertIsInstance(cc, cloud_config.CloudConfig)
+        self.assertTrue(extras.safe_hasattr(cc, 'auth'))
+        self.assertIsInstance(cc.auth, dict)
+        self.assertIsNone(cc.cloud)
+        self.assertIn('username', cc.auth)
+        self.assertEqual('testuser', cc.auth['username'])
+        self.assertEqual('testpass', cc.auth['password'])
+        self.assertFalse(cc.config['image_api_use_tasks'])
+        self.assertTrue('project_name' in cc.auth or 'project_id' in cc.auth)
+        if 'project_name' in cc.auth:
+            self.assertEqual('testproject', cc.auth['project_name'])
+        elif 'project_id' in cc.auth:
+            self.assertEqual('testproject', cc.auth['project_id'])
+        self.assertEqual(cc.region_name, 'region2')
+
     def test_fix_env_args(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
@@ -621,7 +714,7 @@ class TestConfigArgparse(base.TestCase):
                                    vendor_files=[self.vendor_yaml])
 
         cc = c.get_one_cloud(
-            cloud='envvars', argparse=self.options)
+            cloud='envvars', argparse=self.options, validate=False)
         self.assertEqual(cc.auth['project_name'], 'project')
 
     def test_argparse_default_no_token(self):
@@ -648,30 +741,11 @@ class TestConfigArgparse(base.TestCase):
         parser.add_argument('--os-auth-token')
         opts, _remain = parser.parse_known_args(
             ['--os-auth-token', 'very-bad-things',
-             '--os-auth-type', 'token',
-             '--os-auth-url', 'http://example.com/v2',
-             '--os-project-name', 'project'])
-        cc = c.get_one_cloud(argparse=opts)
+             '--os-auth-type', 'token'])
+        cc = c.get_one_cloud(argparse=opts, validate=False)
         self.assertEqual(cc.config['auth_type'], 'token')
         self.assertEqual(cc.config['auth']['token'], 'very-bad-things')
 
-    def test_argparse_username_token(self):
-        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
-                                   vendor_files=[self.vendor_yaml])
-
-        parser = argparse.ArgumentParser()
-        c.register_argparse_arguments(parser, [])
-        # novaclient will add this
-        parser.add_argument('--os-auth-token')
-        opts, _remain = parser.parse_known_args(
-            ['--os-auth-token', 'very-bad-things',
-             '--os-auth-type', 'token',
-             '--os-auth-url', 'http://example.com/v2',
-             '--os-username', 'user',
-             '--os-project-name', 'project'])
-        self.assertRaises(
-            TypeError, c.get_one_cloud, argparse=opts)
-
     def test_argparse_underscores(self):
         c = config.OpenStackConfig(config_files=[self.no_yaml],
                                    vendor_files=[self.no_yaml],
@@ -688,6 +762,17 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(cc.config['auth']['password'], 'pass')
         self.assertEqual(cc.config['auth']['auth_url'], 'auth-url')
 
+    def test_argparse_action_append_no_underscore(self):
+        c = config.OpenStackConfig(config_files=[self.no_yaml],
+                                   vendor_files=[self.no_yaml],
+                                   secure_files=[self.no_yaml])
+        parser = argparse.ArgumentParser()
+        parser.add_argument('--foo', action='append')
+        argv = ['--foo', '1', '--foo', '2']
+        c.register_argparse_arguments(parser, argv=argv)
+        opts, _remain = parser.parse_known_args(argv)
+        self.assertEqual(opts.foo, ['1', '2'])
+
     def test_argparse_underscores_duplicate(self):
         c = config.OpenStackConfig(config_files=[self.no_yaml],
                                    vendor_files=[self.no_yaml],
